import 'dart:collection';

import 'package:aitrainer_app/bloc/customer_change/customer_change_bloc.dart';
import 'package:aitrainer_app/repository/customer_repository.dart';
import 'package:aitrainer_app/util/trans.dart';
import 'package:aitrainer_app/widgets/app_bar_min.dart';
import 'package:aitrainer_app/widgets/app_bar_progress.dart';
import 'package:aitrainer_app/widgets/dialog_html.dart';
import 'package:badges/badges.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

// ignore: must_be_immutable
class CustomerBodyTypePage extends StatefulWidget {
  _CustomerBodyTypePageState _state;

  _CustomerBodyTypePageState createState() {
    _state = _CustomerBodyTypePageState();
    return _state;
  }
}

class BodyTypeItem {
  static String endomorph = "endomorph";
  static String ectomorph = "ectomorph";
  static String mesomorph = "mesomorph";
}

class _CustomerBodyTypePageState extends State<CustomerBodyTypePage> with Trans {
  String selected;
  bool fulldata = false;
  @override
  Widget build(BuildContext context) {
    CustomerRepository customerRepository;
    dynamic args = ModalRoute.of(context).settings.arguments;
    if (args is HashMap && args['personal_data'] != null) {
      fulldata = args['personal_data'];
      customerRepository = args['bloc'];
    } else {
      customerRepository = ModalRoute.of(context).settings.arguments;
    }
    final double cWidth = MediaQuery.of(context).size.width * 0.75;
    setContext(context);

    return Scaffold(
        appBar: fulldata
            ? AppBarMin(
                back: true,
              )
            : AppBarProgress(min: 76, max: 100),
        body: Container(
          decoration: BoxDecoration(
            image: DecorationImage(
              image: AssetImage('asset/image/WT_light_background.jpg'),
              fit: BoxFit.cover,
              alignment: Alignment.center,
            ),
          ),
          child: BlocProvider(
              create: (context) => CustomerChangeBloc(customerRepository: customerRepository),
              child: Builder(builder: (context) {
                // ignore: close_sinks
                CustomerChangeBloc changeBloc = BlocProvider.of<CustomerChangeBloc>(context);

                return Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Divider(),
                    Wrap(
                        //runAlignment: WrapAlignment.center,
                        alignment: WrapAlignment.center,
                        children: [
                          Text(
                            t("Your Body Type"),
                            textAlign: TextAlign.center,
                            style: TextStyle(color: Colors.orange, fontSize: 42, fontFamily: 'Arial', fontWeight: FontWeight.w900),
                          )
                        ]),
                    Divider(),
                    Badge(
                        badgeColor: customerRepository.bodyType == BodyTypeItem.ectomorph ? Colors.orange[200] : Colors.blue[50],
                        badgeContent: GestureDetector(
                          onTap: () => {
                            showDialog(
                                context: context,
                                builder: (BuildContext context) {
                                  return DialogHTML(
                                    title: t("Ectomorph"),
                                    htmlData: t("Ectomorph_desc"),
                                  );
                                })
                          },
                          child: Icon(Icons.info_outline_rounded),
                        ),
                        child: FlatButton(
                            child: Container(
                              width: cWidth,
                              child: Column(
                                children: [
                                  InkWell(
                                    child: Text(
                                      t("Ectomorph"),
                                      style: TextStyle(color: Colors.blue, fontSize: 32, fontFamily: 'Arial', fontWeight: FontWeight.w900),
                                    ),
                                    highlightColor: Colors.white,
                                  ),
                                ],
                              ),
                            ),
                            padding: EdgeInsets.all(10.0),
                            shape: getShape(customerRepository, BodyTypeItem.ectomorph),
                            onPressed: () => {
                                  setState(() {
                                    selected = BodyTypeItem.ectomorph;
                                    changeBloc.add(CustomerBodyTypeChange(bodyType: selected));
                                  }),
                                })),
                    Divider(),
                    Badge(
                        badgeColor: customerRepository.bodyType == BodyTypeItem.endomorph ? Colors.orange : Colors.blue[50],
                        badgeContent: GestureDetector(
                            onTap: () => {
                                  showDialog(
                                      context: context,
                                      builder: (BuildContext context) {
                                        return DialogHTML(
                                          title: t("Endomorph"),
                                          htmlData: t("Endomorph_desc"),
                                        );
                                      })
                                },
                            child: Icon(Icons.info_outline_rounded)),
                        child: FlatButton(
                            child: Container(
                                width: cWidth,
                                child: Column(
                                  children: [
                                    Text(t("Endomorph"),
                                        textWidthBasis: TextWidthBasis.longestLine,
                                        style:
                                            TextStyle(color: Colors.blue, fontSize: 32, fontFamily: 'Arial', fontWeight: FontWeight.w900)),
                                  ],
                                )),
                            padding: EdgeInsets.all(10.0),
                            shape: getShape(customerRepository, BodyTypeItem.endomorph),
                            onPressed: () => {
                                  setState(() {
                                    selected = BodyTypeItem.endomorph;
                                    changeBloc.add(CustomerBodyTypeChange(bodyType: selected));
                                  }),
                                })),
                    Divider(),
                    Badge(
                        badgeColor: customerRepository.bodyType == BodyTypeItem.mesomorph ? Colors.orange[200] : Colors.blue[50],
                        badgeContent: GestureDetector(
                          onTap: () => {
                            showDialog(
                                context: context,
                                builder: (BuildContext context) {
                                  return DialogHTML(
                                    title: t("Mesomorph"),
                                    htmlData: t("Mesomorph_desc"),
                                  );
                                })
                          },
                          child: Icon(Icons.info_outline_rounded),
                        ),
                        child: FlatButton(
                            child: Container(
                              width: cWidth,
                              child: Column(
                                children: [
                                  InkWell(
                                    child: Text(
                                      t("Mesomorph"),
                                      style: TextStyle(color: Colors.blue, fontSize: 32, fontFamily: 'Arial', fontWeight: FontWeight.w900),
                                    ),
                                    highlightColor: Colors.white,
                                  ),
                                ],
                              ),
                            ),
                            padding: EdgeInsets.all(10.0),
                            shape: getShape(customerRepository, BodyTypeItem.mesomorph),
                            onPressed: () => {
                                  setState(() {
                                    selected = BodyTypeItem.mesomorph;
                                    changeBloc.add(CustomerBodyTypeChange(bodyType: selected));
                                  }),
                                })),
                    Divider(),
                    RaisedButton(
                      color: Colors.orange,
                      textColor: Colors.white,
                      child: Text(fulldata ? t("Save") : t("Next")),
                      onPressed: () => {
                        changeBloc.add(CustomerSave()),
                        Navigator.of(context).pop(),
                        if (fulldata == false) {Navigator.of(context).pushNamed("customerWelcomePage", arguments: customerRepository)}
                      },
                    )
                  ],
                );
              })),
        ));
  }

  dynamic getShape(CustomerRepository customerRepository, String fitnessLevel) {
    String selected = customerRepository.bodyType;
    dynamic returnCode = (selected == fitnessLevel)
        ? RoundedRectangleBorder(
            side: BorderSide(width: 4, color: Colors.orange),
          )
        : RoundedRectangleBorder(
            side: BorderSide(width: 1, color: Colors.blue),
          );
    //return
    return returnCode;
  }
}
